#ifndef __CATEGORIES_H__
#define	__CATEGORIES_H__

#include <stdlib.h>
#include <stdbool.h>

struct CategoryEntry {
	char			*name;
	char			*parents[4];
	bool			requires_parent;
};


struct CategoryEntry freedesktop_cat[] = {
	{ "AudioVideo", { NULL }, false },
	{ "Audio", { "AudioVideo", NULL }, true },
	{ "Video", { "AudioVideo", NULL }, true },
	{ "Development", { NULL }, false },
	{ "Education", { NULL }, false },
	{ "Game", { NULL }, false },
	{ "Graphics", { NULL }, false },
	{ "Network", { NULL }, false },
	{ "Office", { NULL }, false },
	{ "Science", { NULL }, false },
	{ "Settings", { NULL }, false },
	{ "System", { NULL }, false },
	{ "Utility", { NULL }, false },
	
	{ "Building", { "Development", NULL }, true },
	{ "Debugger", { "Development", NULL }, true },
	{ "IDE", { "Development", NULL }, true },
	{ "GUIDesigner", { "Development", NULL }, true },
	{ "Profiling", { "Development", NULL }, true },
	{ "RevisionControl", { "Development", NULL }, true },
	{ "Translation", { "Development", NULL }, true },
	{ "Calendar", { "Office", NULL }, true },
	{ "ContactManagement", { "Office", NULL }, true },
	{ "Database", { "Office", "Development", "AudioVideo", NULL }, true },
	{ "Dictionary", { "Office", "TextTools", NULL }, true },
	{ "Chart", { "Office", NULL }, true },
	{ "Email", { "Office", "Network", NULL }, true },
	{ "Finance", { "Office", NULL }, true },
	{ "FlowChart", { "Office", NULL }, true },
	{ "PDA", { "Office", NULL }, true },
	{ "ProjectManagement", { "Office", "Development", NULL }, true },
	{ "Presentation", { "Office", NULL }, true },
	{ "Spreadsheet", { "Office", NULL }, true },
	{ "WordProcessor", { "Office", NULL }, true },
	{ "2DGraphics", { "Graphics", NULL }, true },
	{ "VectorGraphics", { "2DGraphics", NULL }, true },
	{ "RasterGraphics", { "2DGraphics", NULL }, true },
	{ "3DGraphics", { "Graphics", NULL }, true },
	{ "Scanning", { "Graphics", NULL }, true },
	{ "OCR", { "Scanning", NULL }, true },
	{ "Photography", { "Graphics", "Office", NULL }, true },
	{ "Publishing", { "Graphics", "Office", NULL }, true },
	{ "Viewer", { "Graphics", "Office", NULL }, true },
	{ "TextTools", { "Utility", NULL }, true },
	{ "DesktopSettings", { "Settings", NULL }, true },
	{ "HardwareSettings", { "Settings", NULL }, true },
	{ "Printing", { "HardwareSettings", NULL }, true },
	{ "PackageManager", { "Settings", NULL }, true },
	{ "Dialup", { "Network", NULL }, true },
	{ "InstantMessaging", { "Network", NULL }, true },
	{ "Chat", { "Network", NULL }, true },
	{ "IRCClient", { "Network", NULL }, true },
	{ "Feed", { "Network", NULL }, true },
	{ "FileTransfer", { "Network", NULL }, true },
	{ "HamRadio", { "Network", "Audio", NULL }, true },
	{ "News", { "Network", NULL }, true },
	{ "P2P", { "Network", NULL }, true },
	{ "RemoteAccess", { "Network", NULL }, true },
	{ "Telephony", { "Network", NULL }, true },
	{ "TelephonyTools", { "Utility", NULL }, true },
	{ "VideoConferance", { "Network", NULL }, true },
	{ "WebBrowser", { "Network", NULL }, true },
	{ "WebDevelopment", { "Network", "Development", NULL }, true },
	{ "Midi", { "AudioVideo", "Audio", NULL }, true },
	{ "Mixer", { "AudioVideo", "Audio", NULL }, true },
	{ "Tuner", { "AudioVideo", "Audio", NULL }, true },
	{ "TV", { "AudioVideo", "Video", NULL }, true },
	{ "AudioVideoEditing", { "AudioVideo", "Audio", "Video", NULL }, true },
	{ "Player", { "Audio", "Video", "AudioVideo", NULL }, true },
	{ "Recorder", { "Audio", "Video", "AudioVideo", NULL }, true },
	{ "DiscBurning", { "AudioVideo", NULL }, true },
	{ "ActionGame", { "Game", NULL }, true },
	{ "AdventureGame", { "Game", NULL }, true },
	{ "ArcadeGame", { "Game", NULL }, true },
	{ "BoardGame", { "Game", NULL }, true },
	{ "BlocksGame", { "Game", NULL }, true },
	{ "CardGame", { "Game", NULL }, true },
	{ "KidsGame", { "Game", NULL }, true },
	{ "LogicGame", { "Game", NULL }, true },
	{ "RolePlaying", { "Game", NULL }, true },
	{ "Shooter", { "Game", NULL }, true },
	{ "Simulation", { "Game", NULL }, true },
	{ "SportsGame", { "Game", NULL }, true },
	{ "StrategyGame", { "Game", NULL }, true },
	{ "Art", { "Education", "Science", NULL }, true },
	{ "Construction", { "Education", "Science", NULL }, true },
	{ "Music", { "AudioVideo", "Education", NULL}, true },
	{ "Languages", { "Education", "Science", NULL }, true },
	{ "ArtificialIntelligence", { "Education", "Science", NULL }, true },
	{ "Astronomy", { "Education", "Science", NULL }, true },
	{ "Biology", { "Education", "Science", NULL }, true },
	{ "Chemistry", { "Education", "Science", NULL }, true },
	{ "ComputerScience", { "Education", "Science", NULL }, true },
	{ "DataVisualization", { "Education", "Science", NULL }, true },
	{ "Economy", { "Education", "Science", NULL }, true },
	{ "Electricity", { "Education", "Science", NULL }, true },
	{ "Geography", { "Education", "Science", NULL }, true },
	{ "Geology", { "Education", "Science", NULL }, true },
	{ "Geoscience", { "Education", "Science", NULL }, true },
	{ "History", { "Education", "Science", NULL }, true },
	{ "Humanities", { "Education", "Science", NULL }, true },
	{ "ImageProcessing", { "Education", "Science", NULL }, true },
	{ "Literature", { "Education", "Science", NULL }, true },
	{ "Maps", { "Education", "Science", "Utility", NULL }, true },
	{ "Math", { "Education", "Science", NULL }, true },
	{ "NumericalAnalysis", { "Math", NULL }, true },
	{ "MedicalSoftware", { "Education", "Science", NULL }, true },
	{ "Physics", { "Education", "Science", NULL }, true },
	{ "Robotics", { "Education", "Science", NULL }, true },
	{ "Spirituality", { "Education", "Utility", NULL }, true },
	{ "Sports", { "Education", "Science", NULL }, true },
	{ "ParallelComputing", { "ComputerScience", NULL }, true },
	{ "Amusement", { NULL }, true },
	{ "Archiving", { "Utility", NULL }, true },
	{ "Compression", { "Archiving", NULL }, true },
	{ "Electronics", { NULL }, false },
	{ "Emulator", { "System", "Game", NULL }, true },
	{ "Engineering", { NULL }, false },
	{ "FileTools", { "Utility", "System", NULL }, true },
	{ "FileManager", { "FileTools", NULL }, true },
	{ "TerminalEmulator", { "System", NULL }, true },
	{ "Filesystem", { "System", NULL }, true },
	{ "Monitor", { "System", "Network", NULL }, true },
	{ "Security", { "Settings", "System", NULL }, true },
	{ "Accessibility", { "Settings", "Utility", NULL }, true },
	{ "Calculator", { "Utility", NULL }, true },
	{ "Clock", { "Utility", NULL }, true },
	{ "TextEditor", { "Utility", NULL }, true },
	{ "Documentation", { NULL }, false },
	{ "Adult", { NULL }, true },
	{ "Core", { NULL }, true },
	/* TODO: Determine if the desktop specific ones should go here */
};

#endif
